@@ -33,7 +33,7 @@ namespace RIFF {
 // *
 
     /// Returns a human readable path of the given chunk.
-    String __resolveChunkPath(Chunk* pCk) {
+    static String __resolveChunkPath(Chunk* pCk) {
         String sPath;
         for (Chunk* pChunk = pCk; pChunk; pChunk = pChunk->GetParent()) {
             if (pChunk->GetChunkID() == CHUNK_ID_LIST) {
@@ -235,7 +235,7 @@ namespace RIFF {
      *  - RIFF::stream_closed :
      *    the data stream was closed somehow, no more reading possible
      *  - RIFF::stream_end_reached :
-     *    alreaady reached the end of the chunk data, no more reading
+     *    already reached the end of the chunk data, no more reading
      *    possible without SetPos()
      */
     stream_state_t Chunk::GetState() {
@@ -358,7 +358,7 @@ namespace RIFF {
         #elif defined(WIN32)
         if (SetFilePointer(pFile->hFileWrite, ulStartPos + ulPos, NULL/*32 bit*/, FILE_BEGIN) == INVALID_SET_FILE_POINTER) {
             throw Exception("Could not seek to position " + ToString(ulPos) +
-                            " in chunk (" + ToString(ulStartPos + ulPos) + " in file)");            
+                            " in chunk (" + ToString(ulStartPos + ulPos) + " in file)");
         }
         DWORD writtenWords;
         WriteFile(pFile->hFileWrite, pData, WordCount * WordSize, &writtenWords, NULL);
@@ -1268,7 +1268,11 @@ namespace RIFF {
         if (!pSubChunks) {
             pSubChunks    = new ChunkList();
             pSubChunksMap = new ChunkMap();
+            #if defined(WIN32)
+            if (pFile->hFileRead == INVALID_HANDLE_VALUE) return;
+            #else
             if (!pFile->hFileRead) return;
+            #endif
             unsigned long uiOriginalPos = GetPos();
             SetPos(0); // jump to beginning of list chunk body
             while (RemainingBytes() >= CHUNK_HEADER_SIZE) {
@@ -1364,8 +1368,13 @@ namespace RIFF {
      * "from scratch". Note: there must be no empty chunks or empty list
      * chunks when trying to make the new RIFF file persistent with Save()!
      *
+     * Note: by default, the RIFF file will be saved in native endian
+     * format; that is, as a RIFF file on little-endian machines and
+     * as a RIFX file on big-endian. To change this behaviour, call
+     * SetByteOrder() before calling Save().
+     *
      * @param FileType - four-byte identifier of the RIFF file type
-     * @see AddSubChunk(), AddSubList()
+     * @see AddSubChunk(), AddSubList(), SetByteOrder()
      */
     File::File(uint32_t FileType) : List(this) {
         #if defined(WIN32)
@@ -1403,7 +1412,8 @@ namespace RIFF {
                                      path.c_str(), GENERIC_READ,
                                      FILE_SHARE_READ | FILE_SHARE_WRITE,
                                      NULL, OPEN_EXISTING,
-                                     FILE_ATTRIBUTE_NORMAL, NULL
+                                     FILE_ATTRIBUTE_NORMAL |
+                                     FILE_FLAG_RANDOM_ACCESS, NULL
                                  );
         if (hFileRead == INVALID_HANDLE_VALUE) {
             hFileRead = hFileWrite = INVALID_HANDLE_VALUE;
@@ -1456,7 +1466,9 @@ namespace RIFF {
                                                  Filename.c_str(), GENERIC_READ,
                                                  FILE_SHARE_READ | FILE_SHARE_WRITE,
                                                  NULL, OPEN_EXISTING,
-                                                 FILE_ATTRIBUTE_NORMAL, NULL
+                                                 FILE_ATTRIBUTE_NORMAL |
+                                                 FILE_FLAG_RANDOM_ACCESS,
+                                                 NULL
                                              );
                     if (hFileRead == INVALID_HANDLE_VALUE) {
                         hFileRead = hFileWrite = INVALID_HANDLE_VALUE;
@@ -1484,16 +1496,20 @@ namespace RIFF {
                                                  GENERIC_READ | GENERIC_WRITE,
                                                  FILE_SHARE_READ,
                                                  NULL, OPEN_ALWAYS,
-                                                 FILE_ATTRIBUTE_NORMAL, NULL
+                                                 FILE_ATTRIBUTE_NORMAL |
+                                                 FILE_FLAG_RANDOM_ACCESS,
+                                                 NULL
                                              );
                     if (hFileRead == INVALID_HANDLE_VALUE) {
                         hFileRead = hFileWrite = CreateFile(
                                                      Filename.c_str(), GENERIC_READ,
                                                      FILE_SHARE_READ | FILE_SHARE_WRITE,
                                                      NULL, OPEN_EXISTING,
-                                                     FILE_ATTRIBUTE_NORMAL, NULL
+                                                     FILE_ATTRIBUTE_NORMAL |
+                                                     FILE_FLAG_RANDOM_ACCESS,
+                                                     NULL
                                                  );
-                        throw Exception("Could not (re)open file \"" + Filename + "\" in read mode");
+                        throw Exception("Could not (re)open file \"" + Filename + "\" in read+write mode");
                     }
                     #else
                     if (hFileRead) fclose(hFileRead);
@@ -1527,6 +1543,23 @@ namespace RIFF {
         return false;
     }
 
+    /** @brief Set the byte order to be used when saving.
+     *
+     * Set the byte order to be used in the file. A value of
+     * endian_little will create a RIFF file, endian_big a RIFX file
+     * and endian_native will create a RIFF file on little-endian
+     * machines and RIFX on big-endian machines.
+     *
+     * @param Endian - endianess to use when file is saved.
+     */
+    void File::SetByteOrder(endian_t Endian) {
+        #if WORDS_BIGENDIAN
+        bEndianNative = Endian != endian_little;
+        #else
+        bEndianNative = Endian != endian_big;
+        #endif
+    }
+
     /** @brief Save changes to same file.
      *
      * Make all changes of all chunks persistent by writing them to the
@@ -1576,7 +1609,7 @@ namespace RIFF {
             #if defined(WIN32)
             DWORD iBytesMoved = 1; // we have to pass it via pointer to the Windows API, thus the correct size must be ensured
             #else
-            int iBytesMoved = 1; 
+            int iBytesMoved = 1;
             #endif
             for (unsigned long ulPos = 0; iBytesMoved > 0; ulPos += iBytesMoved) {
                 const unsigned long ulToMove = ulFileSize - ulPos;
@@ -1643,7 +1676,8 @@ namespace RIFF {
         #elif defined(WIN32)
         hFileWrite = CreateFile(
                          path.c_str(), GENERIC_WRITE, FILE_SHARE_READ,
-                         NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL
+                         NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL |
+                         FILE_FLAG_RANDOM_ACCESS, NULL
                      );
         if (hFileWrite == INVALID_HANDLE_VALUE) {
             hFileWrite = hFileRead;
@@ -1668,16 +1702,14 @@ namespace RIFF {
         // forget all resized chunks
         ResizedChunks.clear();
 
-        if (Filename.length() > 0) {
-            #if POSIX
-            close(hFileWrite);
-            #elif defined(WIN32)
-            CloseHandle(hFileWrite);
-            #else
-            fclose(hFileWrite);
-            #endif
-            hFileWrite = hFileRead;
-        }
+        #if POSIX
+        if (hFileWrite) close(hFileWrite);
+        #elif defined(WIN32)
+        if (hFileWrite != INVALID_HANDLE_VALUE) CloseHandle(hFileWrite);
+        #else
+        if (hFileWrite) fclose(hFileWrite);
+        #endif
+        hFileWrite = hFileRead;
 
         // associate new file with this File object from now on
         Filename = path;
