@@ -38,6 +38,8 @@
 # define CHUNK_ID_3EWG	0x33657767
 # define CHUNK_ID_EWAV	0x65776176
 # define CHUNK_ID_3GNM	0x33676E6D
+# define CHUNK_ID_EINF	0x65696E66
+# define CHUNK_ID_3CRC	0x33637263
 #else  // little endian
 # define LIST_TYPE_3PRG	0x67727033
 # define LIST_TYPE_3EWL	0x6C776533
@@ -50,6 +52,8 @@
 # define CHUNK_ID_3EWG	0x67776533
 # define CHUNK_ID_EWAV	0x76617765
 # define CHUNK_ID_3GNM	0x6D6E6733
+# define CHUNK_ID_EINF	0x666E6965
+# define CHUNK_ID_3CRC	0x63726333
 #endif // WORDS_BIGENDIAN
 
 /** Gigasampler specific classes and definitions */
@@ -392,23 +396,23 @@ namespace gig {
             // Filter
             bool               VCFEnabled;                    ///< If filter should be used.
             vcf_type_t         VCFType;                       ///< Defines the general filter characteristic (lowpass, highpass, bandpass, etc.).
-            vcf_cutoff_ctrl_t  VCFCutoffController;           ///< Specifies which external controller has influence on the filter cutoff frequency.
+            vcf_cutoff_ctrl_t  VCFCutoffController;           ///< Specifies which external controller has influence on the filter cutoff frequency. @deprecated Don't alter directly, use SetVCFCutoffController() instead!
             bool               VCFCutoffControllerInvert;     ///< Inverts values coming from the defined cutoff controller
             uint8_t            VCFCutoff;                     ///< Max. cutoff frequency.
-            curve_type_t       VCFVelocityCurve;              ///< Defines a transformation curve for the incoming velocity values, affecting the VCF.
-            uint8_t            VCFVelocityScale;              ///< (0-127) Amount velocity controls VCF cutoff frequency (only if no other VCF cutoff controller is defined, otherwise this is the minimum cutoff).
-            uint8_t            VCFVelocityDynamicRange;       ///< 0x04 = lowest, 0x00 = highest
+            curve_type_t       VCFVelocityCurve;              ///< Defines a transformation curve for the incoming velocity values, affecting the VCF. @deprecated Don't alter directly, use SetVCFVelocityCurve() instead!
+            uint8_t            VCFVelocityScale;              ///< (0-127) Amount velocity controls VCF cutoff frequency (only if no other VCF cutoff controller is defined, otherwise this is the minimum cutoff). @deprecated Don't alter directly, use SetVCFVelocityScale() instead!
+            uint8_t            VCFVelocityDynamicRange;       ///< 0x04 = lowest, 0x00 = highest . @deprecated Don't alter directly, use SetVCFVelocityDynamicRange() instead!
             uint8_t            VCFResonance;                  ///< Firm internal filter resonance weight.
             bool               VCFResonanceDynamic;           ///< If <i>true</i>: Increases the resonance Q according to changes of controllers that actually control the VCF cutoff frequency (EG2, ext. VCF MIDI controller).
             vcf_res_ctrl_t     VCFResonanceController;        ///< Specifies which external controller has influence on the filter resonance Q.
             bool               VCFKeyboardTracking;           ///< If <i>true</i>: VCF cutoff frequence will be dependend to the note key position relative to the defined breakpoint value.
             uint8_t            VCFKeyboardTrackingBreakpoint; ///< See VCFKeyboardTracking (0 - 127).
             // Key Velocity Transformations
-            curve_type_t       VelocityResponseCurve;         ///< Defines a transformation curve to the incoming velocity values affecting amplitude (usually you don't have to interpret this parameter, use GetVelocityAttenuation() instead).
-            uint8_t            VelocityResponseDepth;         ///< Dynamic range of velocity affecting amplitude (0 - 4) (usually you don't have to interpret this parameter, use GetVelocityAttenuation() instead).
-            uint8_t            VelocityResponseCurveScaling;  ///< 0 - 127 (usually you don't have to interpret this parameter, use GetVelocityAttenuation() instead)
-            curve_type_t       ReleaseVelocityResponseCurve;  ///< Defines a transformation curve to the incoming release veloctiy values affecting envelope times.
-            uint8_t            ReleaseVelocityResponseDepth;  ///< Dynamic range of release velocity affecting envelope time (0 - 4).
+            curve_type_t       VelocityResponseCurve;         ///< Defines a transformation curve to the incoming velocity values affecting amplitude (usually you don't have to interpret this parameter, use GetVelocityAttenuation() instead). @deprecated Don't alter directly, use SetVelocityResponseCurve() instead!
+            uint8_t            VelocityResponseDepth;         ///< Dynamic range of velocity affecting amplitude (0 - 4) (usually you don't have to interpret this parameter, use GetVelocityAttenuation() instead). @deprecated Don't alter directly, use SetVelocityResponseDepth() instead!
+            uint8_t            VelocityResponseCurveScaling;  ///< 0 - 127 (usually you don't have to interpret this parameter, use GetVelocityAttenuation() instead). @deprecated Don't alter directly, use SetVelocityResponseCurveScaling() instead!
+            curve_type_t       ReleaseVelocityResponseCurve;  ///< Defines a transformation curve to the incoming release veloctiy values affecting envelope times. @deprecated Don't alter directly, use SetReleaseVelocityResponseCurve() instead!
+            uint8_t            ReleaseVelocityResponseDepth;  ///< Dynamic range of release velocity affecting envelope time (0 - 4). @deprecated Don't alter directly, use SetReleaseVelocityResponseDepth() instead!
             uint8_t            ReleaseTriggerDecay;           ///< 0 - 8
             // Mix / Layer
             crossfade_t        Crossfade;
@@ -437,11 +441,26 @@ namespace gig {
             double GetVelocityAttenuation(uint8_t MIDIKeyVelocity);
             double GetVelocityRelease(uint8_t MIDIKeyVelocity);
             double GetVelocityCutoff(uint8_t MIDIKeyVelocity);
+            void SetVelocityResponseCurve(curve_type_t curve);
+            void SetVelocityResponseDepth(uint8_t depth);
+            void SetVelocityResponseCurveScaling(uint8_t scaling);
+            void SetReleaseVelocityResponseCurve(curve_type_t curve);
+            void SetReleaseVelocityResponseDepth(uint8_t depth);
+            void SetVCFCutoffController(vcf_cutoff_ctrl_t controller);
+            void SetVCFVelocityCurve(curve_type_t curve);
+            void SetVCFVelocityDynamicRange(uint8_t range);
+            void SetVCFVelocityScale(uint8_t scaling);
+            Region* GetParent() const;
+            // derived methods
+            DLS::Sampler::AddSampleLoop;
+            DLS::Sampler::DeleteSampleLoop;
             // overridden methods
+            virtual void SetGain(int32_t gain);
             virtual void UpdateChunks();
         protected:
             uint8_t* VelocityTable; ///< For velocity dimensions with custom defined zone ranges only: used for fast converting from velocity MIDI value to dimension bit number.
-            DimensionRegion(RIFF::List* _3ewl);
+            DimensionRegion(Region* pParent, RIFF::List* _3ewl);
+            DimensionRegion(RIFF::List* _3ewl, const DimensionRegion& src);
            ~DimensionRegion();
             friend class Region;
         private:
@@ -480,9 +499,12 @@ namespace gig {
             double*                  pVelocityAttenuationTable;  ///< Points to the velocity table corresponding to the velocity parameters of this DimensionRegion.
             double*                  pVelocityReleaseTable;      ///< Points to the velocity table corresponding to the release velocity parameters of this DimensionRegion
             double*                  pVelocityCutoffTable;       ///< Points to the velocity table corresponding to the filter velocity parameters of this DimensionRegion
+            Region*                  pRegion;
 
             leverage_ctrl_t DecodeLeverageController(_lev_ctrl_t EncodedController);
             _lev_ctrl_t     EncodeLeverageController(leverage_ctrl_t DecodedController);
+            double* GetReleaseVelocityTable(curve_type_t releaseVelocityResponseCurve, uint8_t releaseVelocityResponseDepth);
+            double* GetCutoffVelocityTable(curve_type_t vcfVelocityCurve, uint8_t vcfVelocityDynamicRange, uint8_t vcfVelocityScale, vcf_cutoff_ctrl_t vcfCutoffController);
             double* GetVelocityTable(curve_type_t curveType, uint8_t depth, uint8_t scaling);
             double* CreateVelocityTable(curve_type_t curveType, uint8_t depth, uint8_t scaling);
     };
@@ -496,6 +518,11 @@ namespace gig {
      * will create the mandatory RIFF chunk which will hold the sample wave
      * data and / or resize the file so you will be able to Write() the
      * sample data directly to disk.
+     *
+     * @e Caution: for gig synthesis, most looping relevant information are
+     * retrieved from the respective DimensionRegon instead from the Sample
+     * itself. This was made for allowing different loop definitions for the
+     * same sample under different conditions.
      */
     class Sample : public DLS::Sample {
         public:
@@ -506,14 +533,14 @@ namespace gig {
             uint32_t       FineTune;          ///< Specifies the fraction of a semitone up from the specified MIDI unity note field. A value of 0x80000000 means 1/2 semitone (50 cents) and a value of 0x00000000 means no fine tuning between semitones.
             smpte_format_t SMPTEFormat;       ///< Specifies the Society of Motion Pictures and Television E time format used in the following <i>SMPTEOffset</i> field. If a value of 0 is set, <i>SMPTEOffset</i> should also be set to 0.
             uint32_t       SMPTEOffset;       ///< The SMPTE Offset value specifies the time offset to be used for the synchronization / calibration to the first sample in the waveform. This value uses a format of 0xhhmmssff where hh is a signed value that specifies the number of hours (-23 to 23), mm is an unsigned value that specifies the number of minutes (0 to 59), ss is an unsigned value that specifies the number of seconds (0 to 59) and ff is an unsigned value that specifies the number of frames (0 to -1).
-            uint32_t       Loops;             ///< Number of defined sample loops (so far only seen single loops in gig files - please report me if you encounter more!).
+            uint32_t       Loops;             ///< @e Caution: Use the respective field in the DimensionRegion instead of this one! (Intended purpose: Number of defined sample loops. So far only seen single loops in gig files - please report if you encounter more!)
             uint32_t       LoopID;            ///< Specifies the unique ID that corresponds to one of the defined cue points in the cue point list (only if Loops > 0), as the Gigasampler format only allows one loop definition at the moment, this attribute isn't really useful for anything.
-            loop_type_t    LoopType;          ///< The type field defines how the waveform samples will be looped (only if Loops > 0).
-            uint32_t       LoopStart;         ///< The start value specifies the offset (in sample points) in the waveform data of the first sample to be played in the loop (only if Loops > 0).
-            uint32_t       LoopEnd;           ///< The end value specifies the offset (in sample points) in the waveform data which represents the end of the loop (only if Loops > 0).
-            uint32_t       LoopSize;          ///< Length of the looping area (in sample points) which is equivalent to <i>LoopEnd - LoopStart</i>.
-            uint32_t       LoopFraction;      ///< The fractional value specifies a fraction of a sample at which to loop (only if Loops > 0). This allows a loop to be fine tuned at a resolution greater than one sample. A value of 0 means no fraction, a value of 0x80000000 means 1/2 of a sample length. 0xFFFFFFFF is the smallest fraction of a sample that can be represented.
-            uint32_t       LoopPlayCount;     ///< Number of times the loop should be played (only if Loops > 0, a value of 0 = infinite).
+            loop_type_t    LoopType;          ///< @e Caution: Use the respective field in the DimensionRegion instead of this one! (Intended purpose: The type field defines how the waveform samples will be looped.)
+            uint32_t       LoopStart;         ///< @e Caution: Use the respective field in the DimensionRegion instead of this one! (Intended purpose: The start value specifies the offset [in sample points] in the waveform data of the first sample to be played in the loop [only if Loops > 0].)
+            uint32_t       LoopEnd;           ///< @e Caution: Use the respective field in the DimensionRegion instead of this one! (Intended purpose: The end value specifies the offset [in sample points] in the waveform data which represents the end of the loop [only if Loops > 0].)
+            uint32_t       LoopSize;          ///< @e Caution: Use the respective fields in the DimensionRegion instead of this one! (Intended purpose: Length of the looping area [in sample points] which is equivalent to @code LoopEnd - LoopStart @endcode.)
+            uint32_t       LoopFraction;      ///< The fractional value specifies a fraction of a sample at which to loop. This allows a loop to be fine tuned at a resolution greater than one sample. A value of 0 means no fraction, a value of 0x80000000 means 1/2 of a sample length. 0xFFFFFFFF is the smallest fraction of a sample that can be represented.
+            uint32_t       LoopPlayCount;     ///< Number of times the loop should be played (a value of 0 = infinite).
             bool           Compressed;        ///< If the sample wave is compressed (probably just interesting for instrument and sample editors, as this library already handles the decompression in it's sample access methods anyway).
             uint32_t       TruncatedBits;     ///< For 24-bit compressed samples only: number of bits truncated during compression (0, 4 or 6)
             bool           Dithered;          ///< For 24-bit compressed samples only: if dithering was used during compression with bit reduction
@@ -551,6 +578,7 @@ namespace gig {
             unsigned long        FileNo;                  ///< File number (> 0 when sample is stored in an extension file, 0 when it's in the gig)
             RIFF::Chunk*         pCk3gix;
             RIFF::Chunk*         pCkSmpl;
+            uint32_t             crc;                     ///< CRC-32 checksum of the raw sample data
 
             Sample(File* pFile, RIFF::List* waveList, unsigned long WavePoolOffset, unsigned long fileNo = 0);
            ~Sample();
@@ -593,11 +621,14 @@ namespace gig {
             DimensionRegion*        pDimensionRegions[256];   ///< Pointer array to the 32 (gig2) or 256 (gig3) possible dimension regions (reflects NULL for dimension regions not in use). Avoid to access the array directly and better use GetDimensionRegionByValue() instead, but of course in some cases it makes sense to use the array (e.g. iterating through all DimensionRegions). Use AddDimension() and DeleteDimension() to create a new dimension or delete an existing one (which will create or delete the respective dimension region(s) automatically).
             unsigned int            Layers;                   ///< Amount of defined layers (1 - 32). A value of 1 actually means no layering, a value > 1 means there is Layer dimension. The same information can of course also be obtained by accessing pDimensionDefinitions. Do not alter this value!
 
+            // own methods
             DimensionRegion* GetDimensionRegionByValue(const uint DimValues[8]);
             DimensionRegion* GetDimensionRegionByBit(const uint8_t DimBits[8]);
             Sample*          GetSample();
             void             AddDimension(dimension_def_t* pDimDef);
             void             DeleteDimension(dimension_def_t* pDimDef);
+            // overridden methods
+            virtual void     SetKeyRange(uint16_t Low, uint16_t High);
             virtual void     UpdateChunks();
         protected:
             Region(Instrument* pInstrument, RIFF::List* rgnList);
@@ -632,8 +663,6 @@ namespace gig {
 
             // derived methods from DLS::Resource
             DLS::Resource::GetParent;
-            // derived methods from DLS::Instrument
-            DLS::Instrument::MoveRegion;
             // overridden methods
             Region*   GetFirstRegion();
             Region*   GetNextRegion();
@@ -649,6 +678,7 @@ namespace gig {
            ~Instrument();
             void UpdateRegionKeyTable();
             friend class File;
+            friend class Region; // so Region can call UpdateRegionKeyTable()
     };
 
     /** @brief Group of Gigasampler objects
@@ -687,6 +717,9 @@ namespace gig {
     /** Parses Gigasampler files and provides abstract access to the data. */
     class File : protected DLS::File {
         public:
+            static const DLS::version_t VERSION_2;
+            static const DLS::version_t VERSION_3;
+
             // derived attributes from DLS::Resource
             DLS::Resource::pInfo;
             DLS::Resource::pDLSID;
@@ -716,6 +749,8 @@ namespace gig {
             Group*      AddGroup();
             void        DeleteGroup(Group* pGroup);
             void        DeleteGroupOnly(Group* pGroup);
+            void        SetAutoLoad(bool b);
+            bool        GetAutoLoad();
             virtual    ~File();
             virtual void UpdateChunks();
         protected:
@@ -726,12 +761,14 @@ namespace gig {
             // own protected methods
             virtual void LoadSamples(progress_t* pProgress);
             virtual void LoadInstruments(progress_t* pProgress);
+            void SetSampleChecksum(Sample* pSample, uint32_t crc);
             friend class Region;
             friend class Sample;
             friend class Group; // so Group can access protected member pRIFF
         private:
             std::list<Group*>*          pGroups;
             std::list<Group*>::iterator GroupsIterator;
+            bool                        bAutoLoad;
     };
 
     /**
